#!/usr/bin/env python
#
# Setup script for Berkeley DB XML package
# $Id: setup.py,v 1.65 2004/12/22 17:25:07 gmf Exp $

import os, sys
from distutils.core import setup, Extension

#
# Posix:
#
# db_home, xquery_home, pathan_home, and xerces_home assume
# that these products are installed in their default
# location of /usr/local.  This can be overridden
# with the --with-berkeleydb, --with-xquery, --with-xerces, and --with-pathan
# arguments.
#
# Windows: may require further editing to reflect site specifics.
#
if os.name == "posix":
  db_home = "/usr/local/BerkeleyDB.4.3"
  xerces_home = "/usr/local"
  pathan_home = "/usr/local"
  xquery_home = "/usr/local/xquery-1.0.3"
else:
  xerces_home = "../../../xerces-c-src_2_6_0"
  pathan_home = "../../../pathan"
  xquery_home = "../../../xquery-1.0.3"
  db_home = "../../../db-4.3.27"

for arg in sys.argv:
  if arg.startswith('--with-berkeleydb='):
    db_home = arg.split('=')[1]
  elif arg.startswith('--with-xerces='):
    xerces_home = arg.split('=')[1]
  elif arg.startswith('--with-pathan='):
    pathan_home = arg.split('=')[1]
  elif arg.startswith('--with-xquery='):
    xquery_home = arg.split('=')[1]

sys.argv = filter(lambda x: not x.startswith("--with-"), sys.argv)

debug = "--debug" in sys.argv or "-g" in sys.argv

# setup complains when passed debug flags for install
if "install" in sys.argv and not "build" in sys.argv:
  sys.argv = filter(lambda x: (x != "--debug") and (x != "-g"), sys.argv)

if os.name == "posix":
  INCLUDES =    ["../../include",
                 os.path.join(db_home, "include")]

  LIBDIRS =     ["../../build_unix/.libs",
                 os.path.join(db_home, "lib"),
                 os.path.join(pathan_home, "lib"),
                 os.path.join(xquery_home, "lib"),
                 os.path.join(xerces_home, "lib")]

  LIBS =        ["dbxml-2.0",
                 "db_cxx-4.3",
                 "xquery", "pathan", "xerces-c"]

  DATAFILES =   []

elif os.name == "nt":
  INCLUDES =    ["../../include",
                 os.path.join(db_home, "build_win32"),
                 os.path.join(db_home, "dbinc")]

  if debug:
    LIBDIRS =   ["../../build_win32/Debug",
                 os.path.join(db_home, "build_win32/Debug"),
                 os.path.join(pathan_home, "lib"),
                 os.path.join(xquery_home, "lib"),
                 os.path.join(xerces_home, "Build/Win32/VC7")]
  
    LIBS =      ["libdbxml20D",
                 "libdb43D", "libxquery10d",
                 "@PATHAN_LIBD7", "xerces-c_2D"]

    DATAFILES = [("", ["../../build_win32/Debug/libdbxml20D.dll",
                       os.path.join(db_home, "/build_win32/Debug/libdb43D.dll"),
                       os.path.join(xquery_home, "/lib/libxquery10d.dll"),
                       os.path.join(pathan_home, "/lib/PathanD_7.1.dll"),
                       os.path.join(xerces_home, "/lib/xerces-c_2_6_0D.dll")])]
  else:
    LIBDIRS =   ["../../build_win32/Release",
                 os.path.join(db_home, "build_win32/Release"),
                 os.path.join(xquery_home, "lib"),
                 os.path.join(pathan_home, "lib"),
                 os.path.join(xerces_home, "Build/Win32/VC7")]
  
    LIBS =      ["libdbxml20",
                 "libdb43", "libxquery10",
                 "@PATHAN_LIB7", "xerces-c_2"]

    DATAFILES = [("", ["../../build_win32/Release/libdbxml20.dll",
                       os.path.join(db_home, "build_win32/Release/libdb43.dll"),
                       os.path.join(xquery_home, "/lib/libxquery10.dll"),
                       os.path.join(pathan_home, "/lib/Pathan_7.1.dll"),
                       os.path.join(xerces_home, "/lib/xerces-c_2_6_0.dll")])]

else:
  print "I don't know anything about your platform '%s'." % os.name
  print "Please check the build instructions for more information"
  sys.exit(1)

# Now run with whatever settings we've got
setup(name = "dbxml",
      version = "2.0.9",
      description = "Berkeley DB XML Python API",
      author = "Sleepycat Software",
      author_email = "support@sleepycat.com",
      url = "http://www.sleepycat.com/products/xml.shtml",
      py_modules = ["dbxml"],
      ext_modules = [Extension("_dbxml", ["dbxml_python_wrap.cpp"],
                               include_dirs = INCLUDES,
                               library_dirs = LIBDIRS,
                               libraries = LIBS)],
      data_files = DATAFILES)
