//
// See the file LICENSE for redistribution information.
//
// Copyright (c) 2002-2004
//	Sleepycat Software.  All rights reserved.
//
// $Id: DeleteDocumentCommand.cpp,v 1.3 2004/09/06 14:01:13 jsnelson Exp $
//

#include "DeleteDocumentCommand.hpp"

using namespace DbXml;
using namespace std;

string DeleteDocumentCommand::getCommandName() const
{
  return "deldocument";
}

string DeleteDocumentCommand::getBriefHelp() const
{
  return "Delete a document from the default container";
}

string DeleteDocumentCommand::getMoreHelp() const
{
  return
    string("Takes an argument of the name of the document to delete.") +
    string("This command uses the XmlContainer::deleteDocument() method.\n");
}

void DeleteDocumentCommand::execute(Args &args, Environment &env)
{
  if(args.size() != 2) {
    throw CommandException("Wrong number of arguments");
  }
  env.testContainer();

  if(env.txn()) {
	  XmlTransaction myTxn = env.childTransaction();
	  env.container()->deleteDocument(myTxn, args[1], env.uc());
	  myTxn.commit();
  }
  else {
	  env.container()->deleteDocument(args[1], env.uc());
  }

  if(env.verbose()) cout << "Document deleted, name = " << args[1] << endl;
}

