//
// See the file LICENSE for redistribution information.
//
// Copyright (c) 2002-2004
//	Sleepycat Software.  All rights reserved.
//
// $Id: HelpCommand.cpp,v 1.2 2004/08/19 16:52:27 gareth Exp $
//

#include "HelpCommand.hpp"

using namespace DbXml;
using namespace std;

string HelpCommand::getCommandName() const
{
  return "help";
}

string HelpCommand::getBriefHelp() const
{
  return "Takes an optional command name to print extended help for";
}

string HelpCommand::getMoreHelp() const
{
  return
    string("If no argument is specified, help gives a summary of all the\n") +
    string("commands available.");
}

void HelpCommand::execute(Args &args, Environment &env)
{
  if(args.size() > 2) {
    throw CommandException("Wrong number of arguments");
  }

  vector<ShellCommand*>::iterator end = commands_.end();
  if(args.size() == 1) {

    int maxLength = 4;
    for(vector<ShellCommand*>::iterator i = commands_.begin();
        i != end; ++i) {
      int commandLength = (*i)->getCommandName().length();
      if(commandLength > maxLength) maxLength = commandLength;
    }    

    cout << endl << "Command Summary" << endl;
    cout << "---------------" << endl << endl;
    
    for(vector<ShellCommand*>::iterator j = commands_.begin();
        j != end; ++j) {
      string command = (*j)->getCommandName();
      cout << command << string(maxLength - command.length(), ' ') << " - " <<
        (*j)->getBriefHelp() << endl;
    }
  }
  else {
    vector<ShellCommand*>::iterator i = commands_.begin();
    for(; i != end; ++i) {
      if((*i)->matches(args[1])) {
        string command = (*i)->getCommandName();
        cout << endl << command << endl;
        cout << string(command.length(), '-') << endl;
        cout << (*i)->getBriefHelp() << endl << endl;
        cout << (*i)->getMoreHelp() << endl;
        break;
      }
    }

    if(i == end) {
      throw CommandException("Unknown command: " + args[1]);
    }
  }
}
